@@ -3,12 +3,12 @@
 # This should be sourced from the other scripts
 
 # Make sure TMP_MODVERSION and TMP_STRATEGY are set
-if [ "x$TMP_MODVERSION" = "x" ]; then
+if [ -z "$TMP_MODVERSION" ]; then
 	echo >&2 "ERROR: TMP_MODVERSION not set"
 	exit
 fi
 
-if [ "x$TMP_STRATEGY" = "x" ]; then
+if [ -z "$TMP_STRATEGY" ]; then
 	echo >&2 "ERROR: TMP_STRATEGY not set"
 	exit
 fi
@@ -31,7 +31,7 @@ esac
 
 #Define do_cmd function
 do_cmd()
-{	cmd="$@"
+{	cmd="$*"
 	echo "[$TMP_PROMPT]$ $cmd"
 	$cmd
 }
@@ -42,6 +42,7 @@ case $TMP_STRATEGY in
 	GCCGNU=gnu
 	;;
    *)
+    # shellcheck disable=SC2034
 	GCCGNU=gcc
 	;;
 esac
@@ -57,6 +58,7 @@ case $TMP_MODVERSION in
 		AUTOFLAG=
 		;;
 	   *)
+        # shellcheck disable=SC2034
 		AUTOFLAG="--auto"
 		;;
 	esac
