@@ -36,6 +36,12 @@
 #include "config.h"
 #include "envmodules.h"
 
+/* A specific function named getgrouplist_2 should be used on Darwin systems
+ * to be able to fetch all user groups, especially if user is member of more
+ * then 16 groups */
+#if defined (__APPLE__)
+int32_t getgrouplist_2(const char *, gid_t, gid_t **);
+#endif
 
 /*	Utility function to compare 2 integers for qsort function. */
 int
@@ -71,11 +77,7 @@ Envmodules_GetFilesInDirectoryObjCmd(
 {
    int fetch_dotversion;
    const char *dir;
-#if TCL_MAJOR_VERSION < 9
-   int dirlen;
-#else
    Tcl_Size dirlen;
-#endif
    DIR *did;
    Tcl_Obj *ltmp, *lres;
    struct dirent *direntry;
@@ -190,11 +192,7 @@ Envmodules_ReadFileObjCmd(
    int firstline;
    int must_have_cookie;
    const char *filename;
-#if TCL_MAJOR_VERSION < 9
-   int filenamelen;
-#else
    Tcl_Size filenamelen;
-#endif
    int fid;
    int firstread;
    ssize_t len;
@@ -348,7 +346,6 @@ Envmodules_InitStateUsergroupsObjCmd(
    int objc,               /* Number of arguments. */
    Tcl_Obj *const objv[])  /* Argument objects. */
 {
-   int maxgroups;
    GETGROUPS_T *groups;
    int ngroups = 0;
    int egid_in_groups = 0;
@@ -358,30 +355,34 @@ Envmodules_InitStateUsergroupsObjCmd(
    char gidstr[16];
    Tcl_Obj *lres;
 
-   /* Get actually configured number of groups */
-#if defined(HAVE_SYSCONF) && defined(_SC_NGROUPS_MAX)
-   errno = 0;
-   // just -1 means 'no limit' so errno must checked as well
-   if ((maxgroups = sysconf(_SC_NGROUPS_MAX)) == -1 && errno != 0) {
+#if defined (__APPLE__)
+   uid_t uid;
+   struct passwd *pwd;
+   char uidstr[16];
+
+   /* Fetch user passwd entry */
+   uid = getuid();
+   if ((pwd = getpwuid(uid)) == NULL) {
       Tcl_SetErrno(errno);
+      sprintf (uidstr, "%d", uid);
       Tcl_SetObjResult(interp,
-         Tcl_ObjPrintf("couldn't get NGROUPS_MAX variable: %s",
+         Tcl_ObjPrintf("couldn't find name for user id \"%s\": %s", uidstr,
          Tcl_PosixError(interp)));
       return TCL_ERROR;
    }
+
+   groups = NULL;
+   ngroups = getgrouplist_2(pwd->pw_name, pwd->pw_gid, &groups);
 #else
-#  if defined(NGROUPS_MAX)
-   maxgroups = NGROUPS_MAX;
-#  else
-   maxgroups = DEFAULT_MAXGROUPS;
-#  endif
+#if defined (HAVE_GETGROUPS)
+   ngroups = getgroups(0, (GETGROUPS_T *) NULL);
 #endif
 
    /* Fetch supplementary group list unless getgroups not supported */
-   groups = (GETGROUPS_T *) ckalloc(maxgroups * sizeof(GETGROUPS_T));
+   groups = (GETGROUPS_T *) ckalloc((ngroups + 1) * sizeof(GETGROUPS_T));
 
 #if defined (HAVE_GETGROUPS)
-   if ((ngroups = getgroups(maxgroups, groups)) == -1) {
+   if ((ngroups = getgroups(ngroups, groups)) == -1) {
       Tcl_SetErrno(errno);
       Tcl_SetObjResult(interp,
          Tcl_ObjPrintf("couldn't get supplementary groups: %s",
@@ -389,6 +390,7 @@ Envmodules_InitStateUsergroupsObjCmd(
       ckfree((char *) groups);
       return TCL_ERROR;
    }
+#endif
 #endif
 
    /* Sort then remove duplicates from getgroups result */
@@ -429,7 +431,13 @@ Envmodules_InitStateUsergroupsObjCmd(
          Tcl_SetObjResult(interp,
             Tcl_ObjPrintf("couldn't find name for group id \"%s\": %s",
             gidstr, Tcl_PosixError(interp)));
+         /* groups has been allocated within getgrouplist_2 on Darwin so
+          * memory cannot be released with Tcl-specific free function */
+#if defined (__APPLE__)
+         free((char *) groups);
+#else
          ckfree((char *) groups);
+#endif
          return TCL_ERROR;
       }
       Tcl_ListObjAppendElement(interp, lres, Tcl_NewStringObj(grp->gr_name,
@@ -438,7 +446,11 @@ Envmodules_InitStateUsergroupsObjCmd(
 
    Tcl_SetObjResult(interp, lres);
    Tcl_DecrRefCount(lres);
+#if defined (__APPLE__)
+   free((char *) groups);
+#else
    ckfree((char *) groups);
+#endif
    return TCL_OK;
 }
 
@@ -502,13 +514,8 @@ Envmodules_ParseDateTimeArgObjCmd(
 {
    const char *opt;
    const char *datetime;
-#if TCL_MAJOR_VERSION < 9
-   int optlen;
-   int datetimelen;
-#else
    Tcl_Size optlen;
    Tcl_Size datetimelen;
-#endif
    char dt[17];
    int valid_dt = 0;
    struct tm tm;
@@ -584,8 +591,8 @@ DLLEXPORT int
 Envmodules_Init(
    Tcl_Interp* interp      /* Tcl interpreter */
 ) {
-    /* Require Tcl */
-   if (Tcl_InitStubs(interp, "8.5", 0) == NULL) {
+    /* Require at least Tcl 8.5 */
+   if (Tcl_InitStubs(interp, "8.5-", 0) == NULL) {
       return TCL_ERROR;
    }
 
