@@ -3,7 +3,7 @@
 # MFCMD.TCL, modulefile command procedures
 # Copyright (C) 2002-2004 Mark Lakata
 # Copyright (C) 2004-2017 Kent Mein
-# Copyright (C) 2016-2024 Xavier Delaruelle
+# Copyright (C) 2016-2025 Xavier Delaruelle
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -526,7 +526,7 @@ proc module-tag {args} {
       knerror {No module specified in argument}
    }
    if {$tag in [list loaded auto-loaded forbidden nearly-forbidden hidden\
-      hidden-loaded]} {
+      hidden-loaded warning]} {
       knerror "'$tag' is a reserved tag name and cannot be set"
    }
 
@@ -1143,7 +1143,7 @@ proc parsePrereqCommandArgs {cmd args} {
 
    foreach tag $tag_list {
       if {$tag in [list loaded auto-loaded forbidden nearly-forbidden\
-         hidden]} {
+         hidden warning]} {
          knerror "Tag '$tag' cannot be manually set"
       }
    }
@@ -1486,7 +1486,7 @@ proc execShAndGetEnv {elt_ignored_list shell script args} {
             shell $shdesc] 2>&1; or exit \$status; echo '$sep'; set -xgL;\
             echo '$sep'; $getfunc; string split \$funcout; echo '$sep';\
             string split \$funcout; echo '$sep'; complete; echo '$sep'; pwd"
-         set varre {^(\S+?\M) ?'?(.*?)'?$}
+         set varre {^(\S+?\M) ?['"]?(.*?)['"]?$}
          # exclude alias from function list
          set funcre "^function (\\S+?)(?: \[^\\n\]*?--description\
             (?!'?alias)\[^\\n\]+)?\\n(.+?)?\\s*\\nend\\n$subsep\$"
@@ -2298,6 +2298,90 @@ proc lsb-release {what} {
    }]
 }
 
+proc registerCurrentModuleUse {args} {
+   if {[string length [currentState modulename]]} {
+      setLoadedUse [currentState modulename] {*}$args
+   }
+}
+
+proc module-help {args} {
+   lappend ::g_help_lines [join $args]
+}
+
+proc getModuleHelpLines {} {
+   if {[info exists ::g_help_lines]} {
+      return $::g_help_lines
+   }
+}
+
+# convert property value as module tag (property name is ignored)
+proc add-property {name value} {
+   set tag_list [split $value :]
+   set mod [currentState modulename]
+   foreach tag $tag_list {
+      module-tag $tag $mod
+   }
+}
+
+proc module-warn {args} {
+   # parse application criteria arguments to determine if command apply
+   lassign [parseApplicationCriteriaArgs 1 0 {*}$args] apply isnearly after\
+      otherargs
+
+   # parse remaining argument list, do it even if command does not apply to
+   # raise any command specification error
+   foreach arg $otherargs {
+      if {[info exists nextargisval]} {
+         ##nagelfar vartype nextargisval varName
+         set $nextargisval $arg
+         unset nextargisval
+      } else {
+         switch -glob -- $arg {
+            --message {
+               set nextargisval message
+            }
+            -* {
+               knerror "Invalid option '$arg'"
+            }
+            default {
+               lappend modarglist $arg
+            }
+         }
+         set prevarg $arg
+      }
+   }
+
+   if {[info exists nextargisval]} {
+      knerror "Missing value for '$prevarg' option"
+   }
+   if {![info exists message]} {
+      knerror {No message specified in argument}
+   }
+   if {![info exists modarglist]} {
+      knerror {No module specified in argument}
+   }
+
+   # skip tag record if application criteria are not met
+   if {$apply} {
+      ##nagelfar ignore Found constant
+      set proplist [list message $message]
+      # record each hide spec after parsing them
+      foreach modarg [parseModuleSpecification 0 0 0 0 {*}$modarglist] {
+         setModspecTag $modarg warning $proplist
+      }
+   }
+}
+
+proc provide {args} {
+   if {![llength $args]} {
+      knerror {No module specified in argument}
+   }
+   set current_mod [currentState modulename]
+   foreach alias $args {
+      setLoadedAltname $current_mod [list al $alias]
+   }
+}
+
 # ;;; Local Variables: ***
 # ;;; mode:tcl ***
 # ;;; End: ***
