@@ -3,7 +3,7 @@
 # MODEVAL.TCL, module evaluation procedures
 # Copyright (C) 2002-2004 Mark Lakata
 # Copyright (C) 2004-2017 Kent Mein
-# Copyright (C) 2016-2024 Xavier Delaruelle
+# Copyright (C) 2016-2025 Xavier Delaruelle
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -438,31 +438,20 @@ proc getDirectDependentList {mod {strong 0} {nporeq 0} {loading 0}\
 
    # take currently loading modules into account if asked
    if {$loading} {
-      set modlist [getEnvLoadedModulePropertyParsedList name]
-      defineModEqProc [isIcase] [getConf extended_default] 1
-      # reverse list to get closest match if returning lastly loaded module
-      if {[getConf unload_match_order] eq {returnlast}} {
-         set modlist [lreverse $modlist]
-      }
-      foreach loadingmod [getLoadingModuleList] {
-         foreach prereq [getLoadedPrereq $loadingmod] {
-            set prereq_path_list [getLoadedPrereqPath $loadingmod $prereq]
+      foreach loading_mod [getLoadingModuleList] {
+         foreach prereq_match [getLoadedModulePrereqListAndLoadedMatch\
+            $loading_mod] {
             set lmprelist {}
-            set moddep 0
-            foreach modpre $prereq {
-               foreach lmmod $modlist {
-                  if {[isLoadedMatchSpecificPath $lmmod $prereq_path_list]\
-                     && [modEq $modpre $lmmod eqstart 1 2 1]} {
-                     lappend lmprelist $lmmod
-                     if {$lmmod eq $mod} {
-                        set moddep 1
-                     }
-                     break
-                  }
+            set item_match_list [lassign $prereq_match prereq\
+               prereq_path_list]
+            foreach {prereq_mod_item loaded_mod_match ign} $item_match_list {
+               if {[string length $loaded_mod_match]} {
+                  lappend lmprelist $loaded_mod_match
                }
             }
-            if {$moddep && (!$strong || [llength $lmprelist] == 1)} {
-               lappend deplist $loadingmod
+            if {$mod in $lmprelist && (!$strong || [llength $lmprelist]\
+               == 1)} {
+               lappend deplist $loading_mod
                break
             }
          }
@@ -794,8 +783,10 @@ proc savePropsOfReloadingModule {mod} {
    set conflict_list [getLoadedConflict $mod]
    set prereq_list [getLoadedPrereq $mod]
    set prereq_path_list [getLoadedPrereqPath $mod]
+   set modpath [getModulepathFromLoadedOrLoadingModule $mod]
    set ::g_savedPropsOfReloadMod($mod) [list $is_user_asked $vr_list\
-      $tag_list $extra_tag_list $conflict_list $prereq_list $prereq_path_list]
+      $tag_list $extra_tag_list $conflict_list $prereq_list $prereq_path_list\
+      $modpath]
 }
 
 proc getSavedPropsOfReloadingModule {mod} {
@@ -847,14 +838,15 @@ proc reloadModuleListLoadPhase {mod_list {errmsgtpl {}} {context load}} {
    setConf auto_handling 0
    foreach mod $mod_list {
       lassign [getSavedPropsOfReloadingModule $mod] is_user_asked vr_list\
-         tag_list extra_tag_list conflict_list prereq_list prereq_path_list
+         tag_list extra_tag_list conflict_list prereq_list prereq_path_list\
+         modpath
       # if an auto set default was excluded, module spec need parsing
       lassign [parseModuleSpecification 0 0 0 0 $mod {*}$vr_list] modnamevr
 
       # do not try to reload DepRe module if requirements are not satisfied
       # unless if sticky
       if {$context eq {depre} && ![isModuleLoadable $mod $modnamevr\
-         $conflict_list $prereq_list $prereq_path_list] &&\
+         $conflict_list $prereq_list $prereq_path_list $modpath] &&\
          ![isModuleStickyFromTagList {*}$tag_list {*}$extra_tag_list]} {
          continue
       }
@@ -880,7 +872,7 @@ proc reloadModuleListLoadPhase {mod_list {errmsgtpl {}} {context load}} {
 }
 
 proc isModuleLoadable {mod mod_vr conflict_list prereq_list\
-   prereq_path_list} {
+   prereq_path_list modpath} {
    setLoadedConflict $mod {*}$conflict_list
    set is_conflicting [llength [getModuleLoadedConflict $mod]]
    unsetLoadedConflict $mod
@@ -912,6 +904,13 @@ proc isModuleLoadable {mod mod_vr conflict_list prereq_list\
       }
    }
 
+   # module is loadable if its modulepath is still enabled or if it can be
+   # found in another modulepath (variant availability is currently not
+   # checked by is-avail procedure)
+   if {![is-used $modpath] && ![is-avail $mod_vr]} {
+      return 0
+   }
+
    return 1
 }
 
