@@ -1,7 +1,7 @@
 ##########################################################################
 
 # MODSPEC.TCL, module specification procedures
-# Copyright (C) 2016-2024 Xavier Delaruelle
+# Copyright (C) 2016-2025 Xavier Delaruelle
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -214,11 +214,11 @@ proc getEqArrayKeyProc {arrname name} {
    return $name
 }
 
-# Check a module name does match query at expected depth level when indepth
-# search is disabled. Define procedure on the fly to adapt its
-# code to indepth configuration option and querydepth and test mode params.
-proc defineDoesModMatchAtDepthProc {indepth querydepth test} {
-   set procprops $indepth:$querydepth:$test
+# Check a module name does match query at the depth level expressed in query
+# when search mode is not contains. Define procedure on the fly to adapt its
+# code to search configuration option and querydepth and test mode params.
+proc defineDoesModMatchAtDepthProc {contains querydepth test} {
+   set procprops $contains:$querydepth:$test
 
    # define proc if not done yet or if it was defined for another context
    if {[info procs doesModMatchAtDepth] eq {} ||\
@@ -231,7 +231,7 @@ proc defineDoesModMatchAtDepthProc {indepth querydepth test} {
       set ::g_doesModMatchAtDepth_procprops $procprops
 
       # define optimized procedure
-      if {$indepth} {
+      if {$contains} {
          set atdepth {$mod}
       } else {
          set atdepth "\[join \[lrange \[split \$mod /\] 0 $querydepth\] /\]"
@@ -1146,6 +1146,17 @@ proc modEqProcIcaseExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0}\
    return $ret
 }
 
+proc modEqAny {pattern_list mod {test equal} {trspec 1} {ismodlo 0} {vrcmp 0}\
+   {modvrlist 0} {psuf {}}} {
+   foreach pattern $pattern_list {
+      if {[modEq $pattern $mod $test $trspec $ismodlo $vrcmp $modvrlist\
+         $psuf]} {
+         return 1
+      }
+   }
+   return 0
+}
+
 # analyze module version specified within module specification
 proc parseModuleVersionSpecifier {modspec} {
    set invalidversspec 0
@@ -1300,13 +1311,14 @@ proc parseModuleSpecificationProcAdvVersSpec {mlspec nonamespec xtspec\
    # define extra specifier known list, to raise error if argument does not
    # match
    set xtelt_valid_list [list always-load append-path chdir complete conflict\
-      depends-on envvar family incompat load load-any prepend-path prereq\
-      prereq-all prereq-any pushenv remove-path require set-alias\
-      set-function setenv switch switch-on switch-off tag try-load uncomplete\
-      unload unset-alias unset-function unsetenv variant]
-   set xtelt_modspec_list [list always-load conflict depends-on incompat load\
-      load-any prereq prereq-all prereq-any require switch switch-on\
-      switch-off try-load unload]
+      depends-on depends-on-any envvar family incompat load load-any\
+      prepend-path prereq prereq-all prereq-any provide provided-alias\
+      pushenv remove-path require set-alias set-function setenv switch\
+      switch-on switch-off tag try-load uncomplete unload unset-alias\
+      unset-function unsetenv use variant]
+   set xtelt_modspec_list [list always-load conflict depends-on\
+      depends-on-any incompat load load-any prereq prereq-all prereq-any\
+      require switch switch-on switch-off try-load unload]
 
    set mlunload 0
    set nextmlunload 0
